package com.vmware.psc.alerting.persister.kinesis.producer;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.*;

import com.carbonblack.protocols.psc.response.PSCAlert;
import com.carbonblack.protocols.psc.response.PSCRAugmentedEventBatch;
import com.vmware.psc.alerting.data.uae.UaeAlertCreator;
import com.vmware.psc.alerting.persister.kinesis.producer.properties.KinesisOutputStreamProperties;
import com.vmware.psc.alerting.persister.properties.KinesisConfig;
import com.vmware.psc.alerting.persister.util.CbUaeAlertUtil;
import com.vmware.psc.alerting.sampledata.UaeAlertSamples;
import java.util.Optional;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;

class ProducerSupportTest {

  @Test
  void publishEventBatchWithProducerType() throws Exception {
    ArgumentCaptor<byte[]> captorBatch = ArgumentCaptor.forClass(byte[].class);
    IKinesisProducer iKinesisProducer = mock(KinesisBasicProducer.class);
    KinesisConfig kinesisConfiguration = mock(KinesisConfig.class);
    KinesisOutputStreamProperties kinesisOutputStreamProperties =
        mock(KinesisOutputStreamProperties.class);
    when(kinesisConfiguration.getOutputStream()).thenReturn(kinesisOutputStreamProperties);
    when(kinesisOutputStreamProperties.getStreamName()).thenReturn("test");

    ProducerSupport producerSupport =
        new ProducerSupport(Optional.of(iKinesisProducer), kinesisConfiguration);
    PSCAlert.CbUaeAlert alert = UaeAlertSamples.watchlistIngressAlert();

    producerSupport.publishUaeAlert(alert, CbUaeAlertUtil.getAlertId(alert));

    verify(iKinesisProducer).addRawRecord(any(), captorBatch.capture(), anyString());
    PSCRAugmentedEventBatch.CbAugmentedEventBatchMsg responseBatch =
        PSCRAugmentedEventBatch.CbAugmentedEventBatchMsg.parseFrom(captorBatch.getValue());
    assertNotNull(responseBatch);
    assertEquals(
        PSCRAugmentedEventBatch.CbBatchProducerMetadata.ProducerType.ALERT_PERSISTER,
        responseBatch.getHeader().getProducerMetadata().getProducerType());

    // assert cloud dispatch tag present
    PSCAlert.CbUaeAlert uaeAlert =
        UaeAlertCreator.convertAlerts(responseBatch).stream().findFirst().orElseThrow();
    assertTrue(
        uaeAlert
            .getEvent()
            .getHeader()
            .getCloudDispatchTagsList()
            .contains(
                PSCRAugmentedEventBatch.CbBatchProducerMetadata.ProducerType.ALERT_PERSISTER
                    .name()));
  }

  @Test
  void testDispatchTagNotPresent() throws Exception {
    ArgumentCaptor<byte[]> captorBatch = ArgumentCaptor.forClass(byte[].class);
    IKinesisProducer iKinesisProducer = mock(KinesisBasicProducer.class);
    KinesisConfig kinesisConfiguration = mock(KinesisConfig.class);
    KinesisOutputStreamProperties kinesisOutputStreamProperties =
        mock(KinesisOutputStreamProperties.class);
    when(kinesisConfiguration.getOutputStream()).thenReturn(kinesisOutputStreamProperties);
    when(kinesisOutputStreamProperties.getStreamName()).thenReturn("test");

    ProducerSupport producerSupport =
        new ProducerSupport(Optional.of(iKinesisProducer), kinesisConfiguration);
    PSCAlert.CbUaeAlert alert = UaeAlertSamples.watchlistIngressAlert();
    PSCAlert.CbUaeAlert.Builder builder = PSCAlert.CbUaeAlert.newBuilder(alert);
    builder
        .getEventBuilder()
        .getHeaderBuilder()
        .addCloudDispatchTags(
            PSCRAugmentedEventBatch.CbBatchProducerMetadata.ProducerType.ALERT_PERSISTER.name());

    producerSupport.publishUaeAlert(builder.build(), CbUaeAlertUtil.getAlertId(alert));

    verify(iKinesisProducer).addRawRecord(any(), captorBatch.capture(), anyString());
    PSCRAugmentedEventBatch.CbAugmentedEventBatchMsg responseBatch =
        PSCRAugmentedEventBatch.CbAugmentedEventBatchMsg.parseFrom(captorBatch.getValue());
    assertNotNull(responseBatch);
    assertEquals(
        PSCRAugmentedEventBatch.CbBatchProducerMetadata.ProducerType.ALERT_PERSISTER,
        responseBatch.getHeader().getProducerMetadata().getProducerType());

    // assert cloud dispatch tag present
    PSCAlert.CbUaeAlert uaeAlert =
        UaeAlertCreator.convertAlerts(responseBatch).stream().findFirst().orElseThrow();
    assertTrue(
        uaeAlert
            .getEvent()
            .getHeader()
            .getCloudDispatchTagsList()
            .contains(
                PSCRAugmentedEventBatch.CbBatchProducerMetadata.ProducerType.ALERT_PERSISTER
                    .name()));
  }
}
